/****************************************************************************/
/*  Test DMX512 + MOD-24LR / nrf24L01                                       */
/****************************************************************************/
/*  LEnsE / Julien VILLEMEJANE       /   Institut d'Optique Graduate School */
/*  PROTIS/ Benon Daheron Leprovost Rouabah                                 */
/****************************************************************************/
/*  Programme commandant des canaux DMX selon des octets MIDI               */
/*      Requiert :                                                          */ 
/*            - Nucleo-F767ZI OS 5.14.2                                     */
/*            - Circuit imprimé DMX version F767 avec                       */
/*                un MIDI (IN), un DMX et une SD                            */                   
/****************************************************************************/
/*  Test réalisé sur Nucléo-F767ZI               /!\ Fonctionne pour OS 5.14*/
/****************************************************************************/


#include    "mbed.h"

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>


#include "SDBlockDevice.h"
#include "FATFileSystem.h"


#include    "DMX_MIDI.h"
#include    "projecteurs.h"
#include    "controleurs.h"

// Protocole de communication avec la carte SD
SDBlockDevice   blockDevice(PE_6, PE_5, PE_2, PE_3);  // mosi, miso, sck, cs
FATFileSystem   fileSystem("fs");

Serial          debug_pc(USBTX, USBRX); // Communication série (Teraterm ou autre)
InterruptIn     bp_int(USER_BUTTON); // Inutile pour nous mais utilisé dans du débeugage des librairies DMX/MIDI
DigitalOut      debug_out(D14);


void initialisation_octets(void);

unsigned char* octets2; // Tableau qui contiendra les octets envoyés par les pads WARNIGN PAS LA MEME
int* offset_dmx2; // Tableau qui contiendra les addressages DMX associés aux pads 
int nombre_pad; // Entier qui contiendra le nombre de pads (taille des tableaux précédents)
int nombre_dmx; // Entier qui contiendra le nombre d'adresse DMX (taille des tableaux précédents)
unsigned char dernier_octets; // Retient le dernier octets
// Main
int main() {
    debug_pc.baud(9600);
    int X; // X est EGAL a l'adresse qu'écoute le spot : spot sur d001 --> X = 1
                // Attention la lyre est décalée de 3, donc il être sur d003 au min 
    int curseur[11] = {0,0,0,0,0,0,0,0,0,0,0}; // On dispose de 11 curseurs, les huits et trois potentiomètres
                                               // Sur cette version, les curseurs doivent être associés à 1-11
    int compteur; // servira à parcourir la liste des adressages

    initialisation_octets(); // Création des tableaux
    // Initialisation périphériques
    initDMX();
    initMIDI();
    //initMIDI2();
    //initProjectors();
    //initController();
    
    // Vérification du bon fonctionnement de l'initialisation : 
    // Dans la console, on affiche les trois premiers pads

    while(1) { 
        /* MIDI */
        
        if(isNoteMIDIdetected()){ // La pression sur un pad est détecté
            dernier_octets = note_data;    
            resetNoteMIDI();
        }
        ///*
        if(isCCMIDIdetected()){ // On détecte le déplacement d'un curseur
            // Cela crée deux variables : control_ch octet identifiant le pad ; control_value position du curseur
            printf("Curseur : %x ; Position = %x \r\n",control_ch,control_value);
            if(control_ch == 1){
                curseur[0] = 2 * control_value;
            }
            if(control_ch == 2){
                curseur[1] = 2 * control_value;
            }
            if(control_ch == 3){
                curseur[2] = 2 * control_value;
            }
            if(control_ch == 4){
                curseur[3] = 2 * control_value;
            }
            if(control_ch == 5){
                curseur[4] = 2 * control_value;
            }
            if(control_ch == 6){
                curseur[5] = 2 * control_value;
            }
            if(control_ch == 7){
                curseur[6] = 2 * control_value;
            }            
            if(control_ch == 8){
                curseur[7] = 2 * control_value;
            }

            if(control_ch == 9){
                curseur[8] = 2 * control_value;
            }
            if(control_ch == 10){
                curseur[9] = 2 * control_value;
            }
            if(control_ch == 11){
                curseur[10] = 2 * control_value;
            }
            
            resetCCMIDI();
        }//*/
        
        for (int i = 0; i < nombre_dmx; i++){
            if(dernier_octets == octets2[i]){
                X = offset_dmx2[i];
                dmx_data[X - 1] = curseur[0];
                dmx_data[X + 0] = curseur[1];
                dmx_data[X + 1] = curseur[2];
                dmx_data[X + 2] = curseur[3];
                dmx_data[X + 3] = curseur[4];
                dmx_data[X + 4] = curseur[5];
                dmx_data[X + 5] = curseur[6];
                dmx_data[X + 6] = curseur[7];

                dmx_data[X - 1 - 3] = curseur[8]; // Ces trois curseurs sont censés servir aux lyres
                dmx_data[X + 0 - 3] = curseur[9]; // Ils correspondent donc aux déplacements
                dmx_data[X + 1 - 3] = curseur[10];
            }
        }
        
        updateDMX();
        wait_us(1);
    }
}

void initialisation_octets(){
    //Montage de la carte SD 
    int err = fileSystem.mount(&blockDevice);
    printf("%s\n", (err ? "Fail, montage impossible :(" : "OK"));
    if (!err) {
        // On essaye d'ouvrir le fichier de config
        FILE*   f = fopen("/fs/configM.txt", "r+");
        printf("%s\n", (!f ? "Fail, fichier introuvable :(" : "OK1"));
        if (f) {
            fscanf(f, "%d", &nombre_pad);
            fscanf(f, "%d", &nombre_dmx);
            printf("Nombre pas : %d\r\n",nombre_pad);
            printf("Nombre dmx : %d\r\n",nombre_dmx);
            // Création des tableaux a la bonne taille de octetc2 et offset_dmx2
            octets2 = (unsigned char*) malloc(nombre_dmx*sizeof(unsigned char));
            offset_dmx2 = (int*) malloc(nombre_dmx*sizeof(int));
            int nombre_adresse_du_pad;
            int position_de_remplisage = 0;
            // Go through and increment the numbers
            for (int i = 0; i < nombre_pad; i++) {
                // On lit l'octet et on le sauvegarde dans dernier octets 
                fscanf(f, "%x", &dernier_octets);
                // Pour le nombre de canaux associe a ce pad on recupere leur valeur
                fscanf(f, "%d", &nombre_adresse_du_pad);
                for (int j = 0; j < nombre_adresse_du_pad; j++) {
                    octets2[position_de_remplisage] = dernier_octets;
                    fscanf(f, "%d", &offset_dmx2[position_de_remplisage]);
                    position_de_remplisage++;
                }
            }
        }
        //Verification du remplissage
        printf("Octet 1 : %x ; Octet 2 : %x ; Octet 3 : %x \r\n",octets2[0],octets2[1], octets2[2]);
        printf("Adresse 1 : %d ; Adresse 2 : %d ; Adresse 3 : %d \r\n",offset_dmx2[0],offset_dmx2[1], offset_dmx2[2]);

        // Close the file 
        err = fclose(f);

    }
    // Unmounting SD...
    err = fileSystem.unmount();
    printf("%s\n", (err < 0 ? "Fail, rate le demontage :(" : "OK2"));
}