/****************************************************************************/
/*  Test DMX512 + MOD-24LR / nrf24L01                                       */
/****************************************************************************/
/*  LEnsE / Julien VILLEMEJANE       /   Institut d'Optique Graduate School */
/*  PROTIS/ Benon Daheron Leprovost Rouabah                                 */
/****************************************************************************/
/*  Programme commandant des canaux DMX selon des octets MIDI               */
/*      Requiert :                                                          */ 
/*            - Nucleo-F767ZI OS 5.14.2                                     */
/*            - Circuit imprimé DMX version F767 avec                       */
/*                un MIDI (IN), un DMX et une SD                            */                   
/****************************************************************************/
/*  Test réalisé sur Nucléo-F767ZI               /!\ Fonctionne pour OS 5.14*/
/****************************************************************************/


#include    "mbed.h"

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>

#include "SDBlockDevice.h"
#include "FATFileSystem.h"


#include    "DMX_MIDI.h"
#include    "projecteurs.h"
#include    "controleurs.h"

// Protocole de communication avec la carte SD
SDBlockDevice   blockDevice(PE_6, PE_5, PE_2, PE_3);  // mosi, miso, sck, cs
FATFileSystem   fileSystem("fs");

UnbufferedSerial          debug_pc(USBTX, USBRX); // Communication série (Teraterm ou autre)
InterruptIn     bp_int(BUTTON1); // Inutile pour nous mais utilisé dans du débeugage des librairies DMX/MIDI
DigitalOut      debug_out(D14);

void initialisation_programme_sequence(void);
void lecture_sequence(FILE* fichier); 

unsigned char* octets_PAD_sequences; // Tableau qui contiendra les octets envoyés par les pads utilisées dans le mode sequenciel  
int* numeros_PAD_sequences; // Tableau qui contiendra le numéro de la sequence associé aux PADs
int nombre_sequences; // Entier qui contiendra le nombre de pads (taille des tableaux précédents) associés à une séquence 
int erreur_SD; // permettra de vérifier le bon montage de la carte SD
bool affichage_texte = true;

// Main
int main() {
    debug_pc.baud(9600);
    if(affichage_texte){printf("Lancement du programme \r\n");}

    initialisation_programme_sequence(); // Création des tableaux 
    
    // Initialisation périphériques
    if(affichage_texte){printf("Initialisation peripheriques \r\n");}
    initDMX();
    initMIDI();

    if(affichage_texte){printf("Debut de la boucle While \r\n");}
    while(1) { 

        /* MIDI */
        if(isNoteMIDIdetected()){ // La pression sur un pad est détecté
            //Cela crée deux variables : note_data octet identifiant le pad ; velocity_data pression sur le pad
            if(affichage_texte){printf("Appuie sur le PAD : %x ; force = %x \r\n",note_data,velocity_data);}

            for(int chercheur = 0;chercheur<nombre_sequences;chercheur++){ // On cherche si le pad pressé est dans notre liste
                if(note_data == octets_PAD_sequences[chercheur]){
                    if(affichage_texte){printf("PAD reconnu, on lance la sequence : %d \r\n", numeros_PAD_sequences[chercheur]);}
                    if(numeros_PAD_sequences[chercheur]==1){
                        if(affichage_texte){printf("Ouverture du fichier correspondant a la sequence 1");}
                        FILE* fichier_sequence = fopen("/fs/Sequence1.txt", "r+");
                        printf("%s \r\n", (!fichier_sequence ? "Fail, fichier de sequence introuvable :(" : "OK"));
                        if (fichier_sequence){
                            lecture_sequence(fichier_sequence);
                        }
                        if(affichage_texte){printf("Fermeture du fichier correspondant a la sequence \r\n");}
                        erreur_SD = fclose(fichier_sequence);
                    }
                    if(numeros_PAD_sequences[chercheur]==2){
                        if(affichage_texte){printf("Ouverture du fichier correspondant a la sequence 2");}
                        FILE* fichier_sequence = fopen("/fs/Sequence2.txt", "r+");
                        printf("%s \r\n", (!fichier_sequence ? "Fail, fichier de sequence introuvable :(" : "OK"));
                        if (fichier_sequence){
                            lecture_sequence(fichier_sequence);
                        }
                        if(affichage_texte){printf("Fermeture du fichier correspondant a la sequence \r\n");}
                        erreur_SD = fclose(fichier_sequence);
                    }
                    if(numeros_PAD_sequences[chercheur]==3){
                        if(affichage_texte){printf("Ouverture du fichier correspondant a la sequence 3");}
                        FILE* fichier_sequence = fopen("/fs/Sequence3.txt", "r+");
                        printf("%s \r\n", (!fichier_sequence ? "Fail, fichier de sequence introuvable :(" : "OK"));
                        if (fichier_sequence){
                            lecture_sequence(fichier_sequence);
                        }
                        if(affichage_texte){printf("Fermeture du fichier correspondant a la sequence \r\n");}
                        erreur_SD = fclose(fichier_sequence);
                    }
                    if(numeros_PAD_sequences[chercheur]==4){
                        if(affichage_texte){printf("Ouverture du fichier correspondant a la sequence 4");}
                        FILE* fichier_sequence = fopen("/fs/Sequence4.txt", "r+");
                        printf("%s \r\n", (!fichier_sequence ? "Fail, fichier de sequence introuvable :(" : "OK"));
                        if (fichier_sequence){
                            lecture_sequence(fichier_sequence);
                        }
                        if(affichage_texte){printf("Fermeture du fichier correspondant a la sequence \r\n");}
                        erreur_SD = fclose(fichier_sequence);
                    }
                    if(numeros_PAD_sequences[chercheur]==5){
                        if(affichage_texte){printf("Ouverture du fichier correspondant a la sequence 5");}
                        FILE* fichier_sequence = fopen("/fs/Sequence5.txt", "r+");
                        printf("%s \r\n", (!fichier_sequence ? "Fail, fichier de sequence introuvable :(" : "OK"));
                        if (fichier_sequence){
                            lecture_sequence(fichier_sequence);
                        }
                        if(affichage_texte){printf("Fermeture du fichier correspondant a la sequence \r\n");}
                        erreur_SD = fclose(fichier_sequence);
                    }
                    if(numeros_PAD_sequences[chercheur]==6){
                        if(affichage_texte){printf("Ouverture du fichier correspondant a la sequence 6");}
                        FILE* fichier_sequence = fopen("/fs/Sequence6.txt", "r+");
                        printf("%s \r\n", (!fichier_sequence ? "Fail, fichier de sequence introuvable :(" : "OK"));
                        if (fichier_sequence){
                            lecture_sequence(fichier_sequence);
                        }
                        if(affichage_texte){printf("Fermeture du fichier correspondant a la sequence \r\n");}
                        erreur_SD = fclose(fichier_sequence);
                    }
                    if(numeros_PAD_sequences[chercheur]==7){
                        if(affichage_texte){printf("Ouverture du fichier correspondant a la sequence 7");}
                        FILE* fichier_sequence = fopen("/fs/Sequence7.txt", "r+");
                        printf("%s \r\n", (!fichier_sequence ? "Fail, fichier de sequence introuvable :(" : "OK"));
                        if (fichier_sequence){
                            lecture_sequence(fichier_sequence);
                        }
                        if(affichage_texte){printf("Fermeture du fichier correspondant a la sequence \r\n");}
                        erreur_SD = fclose(fichier_sequence);
                    }
                    if(numeros_PAD_sequences[chercheur]==8){
                        if(affichage_texte){printf("Ouverture du fichier correspondant a la sequence 8");}
                        FILE* fichier_sequence = fopen("/fs/Sequence8.txt", "r+");
                        printf("%s \r\n", (!fichier_sequence ? "Fail, fichier de sequence introuvable :(" : "OK"));
                        if (fichier_sequence){
                            lecture_sequence(fichier_sequence);
                        }
                        if(affichage_texte){printf("Fermeture du fichier correspondant a la sequence \r\n");}
                        erreur_SD = fclose(fichier_sequence);
                    }
                    if(numeros_PAD_sequences[chercheur]==9){
                        if(affichage_texte){printf("Ouverture du fichier correspondant a la sequence 9");}
                        FILE* fichier_sequence = fopen("/fs/Sequence9.txt", "r+");
                        printf("%s \r\n", (!fichier_sequence ? "Fail, fichier de sequence introuvable :(" : "OK"));
                        if (fichier_sequence){
                            lecture_sequence(fichier_sequence);
                        }
                        if(affichage_texte){printf("Fermeture du fichier correspondant a la sequence \r\n");}
                        erreur_SD = fclose(fichier_sequence);
                    }
                    if(numeros_PAD_sequences[chercheur]==10){
                        if(affichage_texte){printf("Ouverture du fichier correspondant a la sequence 10");}
                        FILE* fichier_sequence = fopen("/fs/Sequence10.txt", "r+");
                        printf("%s \r\n", (!fichier_sequence ? "Fail, fichier de sequence introuvable :(" : "OK"));
                        if (fichier_sequence){
                            lecture_sequence(fichier_sequence);
                        }
                        if(affichage_texte){printf("Fermeture du fichier correspondant a la sequence \r\n");}
                        erreur_SD = fclose(fichier_sequence);
                    }
                    if(numeros_PAD_sequences[chercheur]==11){
                        if(affichage_texte){printf("Ouverture du fichier correspondant a la sequence 11");}
                        FILE* fichier_sequence = fopen("/fs/Sequence11.txt", "r+");
                        printf("%s \r\n", (!fichier_sequence ? "Fail, fichier de sequence introuvable :(" : "OK"));
                        if (fichier_sequence){
                            lecture_sequence(fichier_sequence);
                        }
                        if(affichage_texte){printf("Fermeture du fichier correspondant a la sequence \r\n");}
                        erreur_SD = fclose(fichier_sequence);
                    }
                    if(numeros_PAD_sequences[chercheur]==12){
                        if(affichage_texte){printf("Ouverture du fichier correspondant a la sequence 12");}
                        FILE* fichier_sequence = fopen("/fs/Sequence12.txt", "r+");
                        printf("%s \r\n", (!fichier_sequence ? "Fail, fichier de sequence introuvable :(" : "OK"));
                        if (fichier_sequence){
                            lecture_sequence(fichier_sequence);
                        }
                        if(affichage_texte){printf("Fermeture du fichier correspondant a la sequence \r\n");}
                        erreur_SD = fclose(fichier_sequence);
                    }
                    if(numeros_PAD_sequences[chercheur]==13){
                        if(affichage_texte){printf("Ouverture du fichier correspondant a la sequence 13");}
                        FILE* fichier_sequence = fopen("/fs/Sequence13.txt", "r+");
                        printf("%s \r\n", (!fichier_sequence ? "Fail, fichier de sequence introuvable :(" : "OK"));
                        if (fichier_sequence){
                            lecture_sequence(fichier_sequence);
                        }
                        if(affichage_texte){printf("Fermeture du fichier correspondant a la sequence \r\n");}
                        erreur_SD = fclose(fichier_sequence);
                    }
                    if(numeros_PAD_sequences[chercheur]==14){
                        if(affichage_texte){printf("Ouverture du fichier correspondant a la sequence 14");}
                        FILE* fichier_sequence = fopen("/fs/Sequence14.txt", "r+");
                        printf("%s \r\n", (!fichier_sequence ? "Fail, fichier de sequence introuvable :(" : "OK"));
                        if (fichier_sequence){
                            lecture_sequence(fichier_sequence);
                        }
                        if(affichage_texte){printf("Fermeture du fichier correspondant a la sequence \r\n");}
                        erreur_SD = fclose(fichier_sequence);
                    }
                    if(numeros_PAD_sequences[chercheur]==15){
                        if(affichage_texte){printf("Ouverture du fichier correspondant a la sequence 15");}
                        FILE* fichier_sequence = fopen("/fs/Sequence15.txt", "r+");
                        printf("%s \r\n", (!fichier_sequence ? "Fail, fichier de sequence introuvable :(" : "OK"));
                        if (fichier_sequence){
                            lecture_sequence(fichier_sequence);
                        }
                        if(affichage_texte){printf("Fermeture du fichier correspondant a la sequence \r\n");}
                        erreur_SD = fclose(fichier_sequence);
                    }
                    if(numeros_PAD_sequences[chercheur]==16){
                        if(affichage_texte){printf("Ouverture du fichier correspondant a la sequence 16");}
                        FILE* fichier_sequence = fopen("/fs/Sequence16.txt", "r+");
                        printf("%s \r\n", (!fichier_sequence ? "Fail, fichier de sequence introuvable :(" : "OK"));
                        if (fichier_sequence){
                            lecture_sequence(fichier_sequence);
                        }
                        if(affichage_texte){printf("Fermeture du fichier correspondant a la sequence \r\n");}
                        erreur_SD = fclose(fichier_sequence);
                    }
                }
            }       
            resetNoteMIDI();
        }
    }
}

void initialisation_programme_sequence(){
    if(affichage_texte){printf("Lancement de l'initialisation \r\n");}
    if(affichage_texte){printf("Montage de la carte SD \r\n");}
    erreur_SD = fileSystem.mount(&blockDevice);
    printf("%s \r\n", (erreur_SD ? "Fail, montage de la carte SD impossible :(" : "OK"));
    if (!erreur_SD) {
        // On essaye d'ouvrir le fichier contenant les liens séquences PAD
        if(affichage_texte){printf("Ouverture du fichier AdresseSequence.txt \r\n");}
        FILE*   fichier_AdresseSequence = fopen("/fs/AdresseSequence.txt", "r+");
        printf("%s\r\n", (!fichier_AdresseSequence ? "Fail, fichier AdresseSequence.txt introuvable :(" : "OK"));
        if (fichier_AdresseSequence) {
            if(affichage_texte){printf("On recupere le nombre de sequences enregistrees \r\n");}
            fscanf(fichier_AdresseSequence, "%d", &nombre_sequences);
            if(affichage_texte){printf("Nombre Sequences : %d \r\n",nombre_sequences);}

            if(affichage_texte){printf("On creer les tableaux octets_PAD et numeros_PAD a la bonne taille \r\n");}
            octets_PAD_sequences = (unsigned char*) malloc(nombre_sequences*sizeof(unsigned char));
            numeros_PAD_sequences = (int*) malloc(nombre_sequences*sizeof(int));

            // Go through and increment the numbers
            if(affichage_texte){printf("On vient remplir ces tableaux avec les donnees ecrite sur le fichier \r\n");}
            for (int i = 0; i < nombre_sequences; i++) {
                fscanf(fichier_AdresseSequence, "%x", &octets_PAD_sequences[i]);
                fscanf(fichier_AdresseSequence, "%d", &numeros_PAD_sequences[i]);
                if(affichage_texte){printf("Verification de la lecture \r\n");}
                if(affichage_texte){printf(" Octet %d : %x --- Sequence : %d !\r\n", i, octets_PAD_sequences[i], numeros_PAD_sequences[i]);}
            }
        }
        // Close the file
        if(affichage_texte){printf("Fermeture du fichier AdresseSequence.txt \r\n");}
        erreur_SD = fclose(fichier_AdresseSequence);
        //if(affichage_texte){printf("Demontage de la carte SD \r\n");}
        //erreur_SD = fileSystem.unmount();
        //printf("%s\n", (erreur_SD < 0 ? "Fail, rate le demontage :(" : "OK"));
        if(affichage_texte){printf("Fin de l'initialisation \r\n");}
    }
}


void lecture_sequence(FILE* fichier){
    if(affichage_texte){printf("Lancement de la sequence \r\n");}
    int XXX ;   // XXX est EGAL a l'adresse du train dmx que l'on souhaite modifier 
    // Attention cela correspond à la position XXX-1 dans la liste dmx_data
    int AffectedValue; // AffectedValue est EGAL a la value que l'on souhaite affecter à la modification du train dmx
    int Time_ms = 0; // Time_s est EGAL au temps d'attente entre chaque lecture et action en milli-secondes; A la fin du fichier Time_ms = -1 . 
    while(Time_ms != -1) {
        fscanf(fichier, "%d", &XXX);
        fscanf(fichier, "%d", &AffectedValue);
        fscanf(fichier, "%d", &Time_ms);
        if(affichage_texte){printf("Prochaine etape de la sequence : XXX : %d , Value : %d , Time_ms : %d \r\n",XXX,AffectedValue,Time_ms);}
        dmx_data[XXX - 1] = AffectedValue;
        updateDMX();
        if(Time_ms>0){wait_us(1000*Time_ms);}
    }    
    if(affichage_texte){printf("Fin de la sequence");}
}